@@ -1,34 +1,76 @@
-
 --
--- Copyright (c) 2021-2022 Zeping Lee
+-- Copyright (c) 2021-2024 Zeping Lee
 -- Released under the MIT license.
 -- Repository: https://github.com/zepinglee/citeproc-lua
 --
 
 local citation_module = {}
 
-local dom = require("luaxml-domobject")
+local context
+local element
+local ir_node
+local output
+local node_names
+local util
 
-local Context = require("citeproc-context").Context
-local IrState = require("citeproc-context").IrState
-local Element = require("citeproc-element").Element
-local IrNode = require("citeproc-ir-node").IrNode
-local Rendered = require("citeproc-ir-node").Rendered
-local SeqIr = require("citeproc-ir-node").SeqIr
-local YearSuffix = require("citeproc-ir-node").YearSuffix
-local Micro = require("citeproc-output").Micro
-local Formatted = require("citeproc-output").Formatted
-local PlainText = require("citeproc-output").PlainText
-local InlineElement = require("citeproc-output").InlineElement
-local DisamStringFormat = require("citeproc-output").DisamStringFormat
-local SortStringFormat = require("citeproc-output").SortStringFormat
-local util = require("citeproc-util")
+if kpse then
+  context = require("citeproc-context")
+  element = require("citeproc-element")
+  ir_node = require("citeproc-ir-node")
+  output = require("citeproc-output")
+  node_names = require("citeproc-node-names")
+  util = require("citeproc-util")
+else
+  context = require("citeproc.context")
+  element = require("citeproc.element")
+  ir_node = require("citeproc.ir-node")
+  output = require("citeproc.output")
+  node_names = require("citeproc.node-names")
+  util = require("citeproc.util")
+end
+
+local Context = context.Context
+local IrState = context.IrState
+local Element = element.Element
+local IrNode = ir_node.IrNode
+local Rendered = ir_node.Rendered
+local SeqIr = ir_node.SeqIr
+local YearSuffix = ir_node.YearSuffix
+local GroupVar = ir_node.GroupVar
+local Micro = output.Micro
+local Formatted = output.Formatted
+local PlainText = output.PlainText
+local InlineElement = output.InlineElement
+local CiteInline = output.CiteInline
+local DisamStringFormat = output.DisamStringFormat
+local SortStringFormat = output.SortStringFormat
+
+local Position = util.Position
 
 
+-- local DEBUG_DISAMBIGUATE = true
+
+
+---@class Citation: Element
+---@field disambiguate_add_givenname boolean?
+---@field givenname_disambiguation_rule string
+---@field disambiguate_add_names boolean?
+---@field disambiguate_add_year_suffix boolean?
+---@field cite_group_delimiter string
+---@field cite_grouping boolean?
+---@field collapse string?
+---@field year_suffix_delimiter string
+---@field after_collapse_delimiter string
+---@field near_note_distance integer
+---@field style Style
+---@field sort Sort
+---@field layout Layout
+---@field layouts_by_language { [string]: Layout }
+---@field name_inheritance Name
 local Citation = Element:derive("citation", {
   givenname_disambiguation_rule = "by-cite",
   -- https://github.com/citation-style-language/schema/issues/338
-  -- The cite_group_delimiter may be changed to inherit the delimiter in citaion > layout.
+  -- The cite_group_delimiter may be changed to inherit the delimiter in citation > layout.
   cite_group_delimiter = ", ",
   near_note_distance = 5,
 })
@@ -37,6 +79,9 @@ function Citation:from_node(node, style)
 
   local o = self:new()
   o.children = {}
+  o.style = style
+  o.layout = nil
+  o.layouts_by_language = {}
 
   o:process_children_nodes(node)
 
@@ -45,7 +90,13 @@ function Citation:from_node(node, style)
   for _, child in ipairs(o.children) do
     local element_name = child.element_name
     if element_name == "layout" then
-      o.layout = child
+      if child.locale then
+        for _, lang in ipairs(util.split(util.strip(child.locale))) do
+          o.layouts_by_language[lang] = child
+        end
+      else
+        o.layout = child
+      end
     elseif element_name == "sort" then
       o.sort = child
     end
@@ -85,7 +136,7 @@ function Citation:from_node(node, style)
   -- Note Distance
   o:set_number_attribute(node, "near-note-distance")
 
-  local name_inheritance = require("citeproc-node-names").Name:new()
+  local name_inheritance = node_names.Name:new()
   for key, value in pairs(style.name_inheritance) do
     if value ~= nil then
       name_inheritance[key] = value
@@ -99,6 +150,9 @@ function Citation:from_node(node, style)
   return o
 end
 
+---@param citation CitationData
+---@param engine CiteProc
+---@return string
 function Citation:build_citation_str(citation, engine)
   if engine.registry.requires_sorting then
     engine:sort_bibliography()
@@ -111,7 +165,7 @@ end
 -- Formatting is stripped from the author-only and composite renderings
 -- of the author name
 local function remove_name_formatting(ir)
-  if ir._element == "name" then
+  if ir._element_name == "name" then
     ir.formatting = nil
   end
   if ir.children then
@@ -121,6 +175,22 @@ local function remove_name_formatting(ir)
   end
 end
 
+---@alias CiteId string | number
+
+---@class CitationItem
+---@field id CiteId
+---@field prefix string?
+---@field suffix string?
+---@field locator string?
+---@field label string?
+---@field prefix_inlines InlineElement[]?
+---@field suffix_inlines InlineElement[]?
+
+---comment
+---@param citation_items CitationItem[]
+---@param engine CiteProc
+---@param properties table
+---@return string
 function Citation:build_cluster(citation_items, engine, properties)
   properties = properties or {}
   local output_format = engine.output_format
@@ -156,11 +226,15 @@ function Citation:build_cluster(citation_items, engine, properties)
       -- if layout_affixes then
       --   layout_prefix = layout_affixes.prefix
       -- end
-    local prefix = citation_items[i].prefix
-    if prefix then
+    local prefix_inlines = citation_items[i].prefix_inlines
+    if prefix_inlines then
       -- Prefix is inlines
-      local right_most_str = prefix[#prefix]:get_right_most_string()
-      if string.match(right_most_str, "[.!?]%s*$") and InlineElement.has_space(prefix) then
+      local prefix_str = output.SortStringFormat:new():output(prefix_inlines, context)
+      if (string.match(prefix_str, "[.!?]%s*$")
+          -- position_IbidWithPrefixFullStop.txt
+          -- `Book A. He said “Please work.” Ibid.`
+          or string.match(prefix_str, "[.!?]”%s*$")
+         ) and InlineElement.has_space(prefix_inlines) then
         ir:capitalize_first_term()
       end
     else
@@ -173,7 +247,6 @@ function Citation:build_cluster(citation_items, engine, properties)
 
   -- util.debug(irs)
 
-  local citation_delimiter = self.layout.delimiter
   local citation_stream = {}
 
   local context = Context:new()
@@ -187,23 +260,25 @@ function Citation:build_cluster(citation_items, engine, properties)
 
   local previous_ir
   for i, ir in ipairs(irs) do
-    local cite_prefix = citation_items[i].prefix
-    local cite_suffix = citation_items[i].suffix
+    local cite_prefix = citation_items[i].prefix_inlines
+    local cite_suffix = citation_items[i].suffix_inlines
     if not ir.collapse_suppressed then
-      local ir_inlines = ir:flatten(output_format)
-      if #ir_inlines > 0 then
-        -- Make sure ir_inlines has outputs contents.
+      local cite_inlines = ir:flatten(output_format)
+      if #cite_inlines > 0 then
+        -- Make sure cite_inlines has outputs contents.
         -- collapse_AuthorCollapseNoDateSorted.txt
         if previous_ir then
-          if previous_ir.own_delimiter then
-            table.insert(citation_stream, PlainText:new(previous_ir.own_delimiter))
-          else
-            local left_most_str
+          local delimiter = previous_ir.own_delimiter or previous_ir.cite_delimiter
+          if delimiter then
             if cite_prefix then
+              local left_most_str
               left_most_str = cite_prefix[1]:get_left_most_string()
+              if string.match(left_most_str, "^[,.;?!]") then
+                delimiter = nil
+              end
             end
-            if citation_delimiter and not (cite_prefix and util.startswith(left_most_str, ",")) then
-              table.insert(citation_stream, PlainText:new(citation_delimiter))
+            if delimiter then
+              table.insert(citation_stream, PlainText:new(delimiter))
             end
           end
         end
@@ -212,17 +287,32 @@ function Citation:build_cluster(citation_items, engine, properties)
           table.insert(citation_stream, Micro:new(cite_prefix))
         end
 
-        -- util.debug(ir)
-        util.extend(citation_stream, ir_inlines)
-        previous_ir = ir
+        -- util.debug(ir.cite_item.id)
+        -- util.debug(cite_inlines)
+        -- if context.engine.opt.citation_link then
+          cite_inlines = {CiteInline:new(cite_inlines, ir.cite_item)}
+        -- end
+        util.extend(citation_stream, cite_inlines)
 
         if cite_suffix then
           table.insert(citation_stream, Micro:new(cite_suffix))
+          local cite_delimiter = ir.own_delimiter or ir.cite_delimiter
+          if cite_delimiter and string.match(cite_delimiter, "^[,.;?]") then
+            -- affix_WithCommas.txt
+            local right_most_str = cite_suffix[#cite_suffix]:get_right_most_string()
+            if string.match(right_most_str, "[,.;?]%s*$") then
+              ir.own_delimiter = string.gsub(cite_delimiter,  "^[,.;?]", "")
+            end
+          end
         end
+
+        previous_ir = ir
       end
     end
   end
 
+  -- util.debug(citation_stream)
+
   local has_printed_form = true
   if #citation_items == 0 then
     -- bugreports_AuthorOnlyFail.txt
@@ -232,19 +322,19 @@ function Citation:build_cluster(citation_items, engine, properties)
     -- date_DateNoDateNoTest.txt
     has_printed_form = false
     citation_stream = {PlainText:new("[CSL STYLE ERROR: reference with no printed form.]")}
-  elseif #citation_stream == 1 and citation_stream[1].value == "[NO_PRINTED_FORM]" then
+  elseif #citation_stream == 1 and citation_stream[1]._type == "CiteInline" and
+      #citation_stream[1].inlines == 1 and citation_stream[1].inlines[1].value == "[NO_PRINTED_FORM]" then
     has_printed_form = false
   end
 
   local author_only_mode = (properties.mode == "author-only" or
     (#citation_items >= 1 and citation_items[1]["author-only"]))
   if has_printed_form and context.area.layout.affixes and not author_only_mode then
-    local affixes = context.area.layout.affixes
-    if affixes.prefix then
-      table.insert(citation_stream, 1, PlainText:new(affixes.prefix))
+    if irs[1].cite_prefix then
+      table.insert(citation_stream, 1, PlainText:new(irs[1].cite_prefix))
     end
-    if affixes.suffix then
-      table.insert(citation_stream, PlainText:new(affixes.suffix))
+    if irs[#irs].cite_suffix then
+      table.insert(citation_stream, PlainText:new(irs[#irs].cite_suffix))
     end
   end
 
@@ -271,7 +361,7 @@ function Citation:build_cluster(citation_items, engine, properties)
           -- discretionary_AuthorOnlySuppressLocator.txt
           infix = " "
         end
-        for i, inline in ipairs(InlineElement:parse(infix, context)) do
+        for i, inline in ipairs(InlineElement:parse(infix, context, true)) do
           table.insert(citation_stream, i, inline)
         end
       else
@@ -314,24 +404,58 @@ function Citation:sorted_citation_items(items, engine)
   return items
 end
 
+
+---@class CiteIr: IrNode
+---@field cite_item CitationItem
+---@field reference table
+---@field ir_index any
+---@field is_ambiguous boolean
+---@field disam_level integer
+---@field disam_str string?
+---@field cite_delimiter string?
+---@field cite_prefix string?
+---@field cite_suffix string?
+
+---@param cite_item CitationItem
+---@param output_format OutputFormat
+---@param engine CiteProc
+---@param properties table
+---@return CiteIr
 function Citation:build_fully_disambiguated_ir(cite_item, output_format, engine, properties)
+  -- util.debug(cite_item.id)
   local cite_ir = self:build_ambiguous_ir(cite_item, output_format, engine)
-  -- util.debug(cite_ir)
   cite_ir = self:apply_disambiguate_add_givenname(cite_ir, engine)
   cite_ir = self:apply_disambiguate_add_names(cite_ir, engine)
   cite_ir = self:apply_disambiguate_conditionals(cite_ir, engine)
   cite_ir = self:apply_disambiguate_add_year_suffix(cite_ir, engine)
 
+  if engine.style.class == "note" and cite_item.position_level == Position.First then
+    -- Disambiguation should be based on the subsequent form
+    -- disambiguate_BasedOnEtAlSubsequent.txt
+    cite_item.position_level = Position.Subsequent
+    local disam_ir = self:build_ambiguous_ir(cite_item, output_format, engine)
+    disam_ir = self:apply_disambiguate_add_givenname(disam_ir, engine)
+    disam_ir = self:apply_disambiguate_add_names(disam_ir, engine)
+    disam_ir = self:apply_disambiguate_conditionals(disam_ir, engine)
+    disam_ir = self:apply_disambiguate_add_year_suffix(disam_ir, engine)
+    cite_item.position_level = Position.First
+  end
+
   return cite_ir
 end
 
+---comment
+---@param cite_item CitationItem
+---@param output_format OutputFormat
+---@param engine CiteProc
+---@return CiteIr
 function Citation:build_ambiguous_ir(cite_item, output_format, engine)
   local state = IrState:new(engine.style)
   local context = Context:new()
   context.engine = engine
   context.style = engine.style
   context.area = self
-  context.locale = engine:get_locale(engine.lang)
+  -- context.locale = engine:get_locale(engine.lang)
   context.name_inheritance = self.name_inheritance
   context.format = output_format
   context.id = cite_item.id
@@ -339,11 +463,14 @@ function Citation:build_ambiguous_ir(cite_item, output_format, engine)
   -- context.reference = self:get_item(cite_item.id)
   context.reference = engine.registry.registry[cite_item.id]
 
+  local active_layout, context_lang = util.get_layout_by_language(self, engine, context.reference)
+  context.locale = engine:get_locale(context_lang)
+
   local ir
   if context.reference then
-    ir = self:build_ir(engine, state, context)
+    ir = self:build_ir(engine, state, context, active_layout)
   else
-    ir = Rendered:new({Formatted:new({PlainText:new(cite_item.id)}, {["font-weight"] = "bold"})}, self)
+    ir = Rendered:new({Formatted:new({PlainText:new(tostring(cite_item.id))}, {["font-weight"] = "bold"})}, self)
   end
 
   ir.cite_item = cite_item
@@ -353,6 +480,12 @@ function Citation:build_ambiguous_ir(cite_item, output_format, engine)
   ir.is_ambiguous = false
   ir.disam_level = 0
 
+  ir.cite_delimiter = active_layout.delimiter
+  if active_layout.affixes then
+    ir.cite_prefix = active_layout.affixes.prefix
+    ir.cite_suffix = active_layout.affixes.suffix
+  end
+
   -- Formattings like font-style are ignored for disambiguation.
   local disam_format = DisamStringFormat:new()
   local inlines = ir:flatten(disam_format)
@@ -369,21 +502,40 @@ function Citation:build_ambiguous_ir(cite_item, output_format, engine)
       break
     end
   end
+  if DEBUG_DISAMBIGUATE then
+    if ir.is_ambiguous then
+      util.debug("[CLASH]")
+      util.debug(string.format("%s: %s", cite_item.id, disam_str))
+      for ir_index, ir_ in pairs(engine.cite_irs_by_output[disam_str]) do
+        util.debug(string.format("%s: %s", ir_.cite_item.id, ir_.disam_str))
+      end
+    else
+      util.debug("[clear]")
+      util.debug(string.format("%s: %s", cite_item.id, disam_str))
+    end
+  end
   engine.cite_irs_by_output[disam_str][ir.ir_index] = ir
 
   return ir
 end
 
-function Citation:build_ir(engine, state, context)
-  if not self.layout then
+---@param engine CiteProc
+---@param state State
+---@param context Context
+---@param active_layout Layout
+---@return CiteIr
+function Citation:build_ir(engine, state, context, active_layout)
+  if not active_layout then
     util.error("Missing citation layout.")
   end
-  return self.layout:build_ir(engine, state, context)
+  return active_layout:build_ir(engine, state, context)
 end
 
 function Citation:apply_disambiguate_add_givenname(cite_ir, engine)
   if self.disambiguate_add_givenname then
-    -- util.debug("disambiguate_add_givenname: " .. cite_ir.cite_item.id)
+    if DEBUG_DISAMBIGUATE then
+      util.debug("[Method (1)] disambiguate-add-givenname: " .. cite_ir.cite_item.id)
+    end
 
     local gn_disam_rule = self.givenname_disambiguation_rule
     if gn_disam_rule == "all-names" or gn_disam_rule == "all-names-with-initials" then
@@ -565,9 +717,11 @@ function Citation:apply_disambiguate_add_givenname_primary_name(cite_ir, engine)
 end
 
 function Citation:apply_disambiguate_add_givenname_by_cite(cite_ir, engine)
+  -- util.debug(cite_ir.is_ambiguous)
   if not cite_ir.is_ambiguous then
     return cite_ir
   end
+  -- util.debug(cite_ir)
   if not cite_ir.person_name_irs or #cite_ir.person_name_irs == 0 then
     return cite_ir
   end
@@ -699,14 +853,16 @@ function Citation:apply_disambiguate_add_names(cite_ir, engine)
     return cite_ir
   end
 
-  -- util.debug("disambiguate_add_names: " .. cite_ir.cite_item.id)
-
-  if name_ir then
-    -- util.debug(cite_ir.disam_str)
-    -- util.debug(cite_ir.name_ir.full_name_str)
-    -- util.debug(cite_ir.is_ambiguous)
+  if DEBUG_DISAMBIGUATE then
+    util.debug("[Method (3)] disambiguate-add-names: " .. cite_ir.cite_item.id)
   end
 
+  -- if name_ir then
+  --   util.debug(cite_ir.disam_str)
+  --   util.debug(cite_ir.name_ir.full_name_str)
+  --   util.debug(cite_ir.is_ambiguous)
+  -- end
+
   local disam_format = DisamStringFormat:new()
 
   while cite_ir.is_ambiguous do
@@ -811,7 +967,9 @@ function Citation:collect_irs_with_disambiguate_branch(ir)
 end
 
 function Citation:apply_disambiguate_conditionals(cite_ir, engine)
-  -- util.debug(cite_ir)
+  if DEBUG_DISAMBIGUATE then
+    util.debug("[Method (3)] disambiguate with condition testing “true”: " .. cite_ir.cite_item.id)
+  end
 
   cite_ir.irs_with_disambiguate_branch = self:collect_irs_with_disambiguate_branch(cite_ir)
 
@@ -837,8 +995,10 @@ function Citation:apply_disambiguate_conditionals(cite_ir, engine)
       if #ir_.irs_with_disambiguate_branch > 0 then
         -- Disambiguation is incremental
         -- disambiguate_IncrementalExtraText.txt
+        ---@type SeqIr
         local condition_ir = ir_.irs_with_disambiguate_branch[1]
         condition_ir.children[1] = condition_ir.disambiguate_branch_ir
+        -- condition_ir.person_name_irs are no longer used and we ignore them
         condition_ir.group_var = condition_ir.disambiguate_branch_ir.group_var
         table.remove(ir_.irs_with_disambiguate_branch, 1)
         -- disambiguate_DisambiguateTrueReflectedInBibliography.txt
@@ -867,22 +1027,48 @@ function Citation:apply_disambiguate_conditionals(cite_ir, engine)
 end
 
 function Citation:check_ambiguity(cite_ir, engine)
+  local is_ambiguous = false
   for _, ir_ in pairs(engine.cite_irs_by_output[cite_ir.disam_str]) do
     if ir_.cite_item.id ~= cite_ir.cite_item.id then
-      return true
+      is_ambiguous = true
     end
   end
-  return false
+  if DEBUG_DISAMBIGUATE then
+    if is_ambiguous then
+      util.debug("[CLASH]")
+      util.debug(string.format("%s, %s", cite_ir.cite_item.id, cite_ir.disam_str))
+      for _, ir_ in pairs(engine.cite_irs_by_output[cite_ir.disam_str]) do
+        if ir_.cite_item.id ~= cite_ir.cite_item.id then
+          util.debug(string.format("%s: %s", ir_.cite_item.id, ir_.disam_str))
+        end
+      end
+    else
+      util.debug("[clear]")
+      util.debug(string.format("%s, %s", cite_ir.cite_item.id, cite_ir.disam_str))
+    end
+  end
+  return is_ambiguous
 end
 
-function Citation:get_same_output_irs(cite_ir, engine)
-  local ambiguous_same_output_irs = {}
-  for _, ir_ in pairs(engine.cite_irs_by_output[cite_ir.disam_str]) do
-    if ir_.disam_str == cite_ir.disam_str then
-      table.insert(ambiguous_same_output_irs, ir_)
+function Citation:get_ambiguous_cite_irs(cite_ir, engine)
+  local res = {}
+  for _, ir in pairs(engine.cite_irs_by_output[cite_ir.disam_str]) do
+    if ir.cite_item.id ~= cite_ir.cite_item.id then
+      table.insert(res, ir)
     end
   end
-  return ambiguous_same_output_irs
+  return res
+end
+
+function Citation:get_ambiguous_same_output_cite_irs(cite_ir, engine)
+  -- This includes the cite_ir itself.
+  local res = {}
+  for _, ir in pairs(engine.cite_irs_by_output[cite_ir.disam_str]) do
+    if ir.disam_str == cite_ir.disam_str then
+      table.insert(res, ir)
+    end
+  end
+  return res
 end
 
 function Citation:apply_disambiguate_add_year_suffix(cite_ir, engine)
@@ -890,60 +1076,67 @@ function Citation:apply_disambiguate_add_year_suffix(cite_ir, engine)
     return cite_ir
   end
 
-  local same_output_irs = self:get_same_output_irs(cite_ir, engine)
+  if DEBUG_DISAMBIGUATE then
+    util.debug("[Method (4)] disambiguate-add-year-suffix”: " .. cite_ir.cite_item.id)
+  end
 
-  table.sort(same_output_irs, function (a, b)
+  local ambiguous_same_output_irs = self:get_ambiguous_same_output_cite_irs(cite_ir, engine)
+
+  table.sort(ambiguous_same_output_irs, function (a, b)
     -- return a.ir_index < b.ir_index
     return a.reference["citation-number"] < b.reference["citation-number"]
   end)
 
-  local year_suffix_number = 0
-  -- util.debug(cite_ir)
-
   local disam_format = DisamStringFormat:new()
 
-  for _, ir_ in ipairs(same_output_irs) do
+  for _, ir_ in ipairs(ambiguous_same_output_irs) do
     ir_.reference.year_suffix_number = nil
   end
 
-  for _, ir_ in ipairs(same_output_irs) do
-    -- print(ir_.cite_item.id)
-    -- print(ir_.reference)
+  -- TODO: clear year-suffix after updateItems
+  local year_suffix_number = 0
+  for _, ir_ in ipairs(ambiguous_same_output_irs) do
     if not ir_.reference.year_suffix_number then
       year_suffix_number = year_suffix_number + 1
       ir_.reference.year_suffix_number = year_suffix_number
       ir_.reference["year-suffix"] = self:render_year_suffix(year_suffix_number)
     end
+    -- util.debug(string.format('%s: %s "%s"', ir_.cite_item.id, tostring(ir_.reference.year_suffix_number), ir_.reference["year-suffix"]))
 
     if not ir_.year_suffix_irs then
       ir_.year_suffix_irs = ir_:collect_year_suffix_irs()
       if #ir_.year_suffix_irs == 0 then
-        -- By default, the year-suffix is appended the first year rendered through cs:date
+        -- The style does not have a "year-suffix" variable.
+        -- Then the year-suffix is appended the first year rendered through cs:date or citation-label
         local year_ir = ir_:find_first_year_ir()
         -- util.debug(year_ir)
         if year_ir then
           local year_suffix_ir = YearSuffix:new({}, self)
           table.insert(year_ir.children, year_suffix_ir)
           table.insert(ir_.year_suffix_irs, year_suffix_ir)
+        else
+          util.warning("No date variable for year-suffix")
         end
       end
     end
 
+    -- Render all the year-suffix irs.
     for _, year_suffix_ir in ipairs(ir_.year_suffix_irs) do
-      year_suffix_ir.inlines = {PlainText:new(ir_.reference["year-suffix"])}
+      -- util.debug(ir_.reference["year-suffix"])
+      year_suffix_ir.inlines = { PlainText:new(ir_.reference["year-suffix"]) }
       year_suffix_ir.year_suffix_number = ir_.reference.year_suffix_number
-      year_suffix_ir.group_var = "important"
+      year_suffix_ir.group_var = GroupVar.Important
     end
 
-    local inlines = ir_:flatten(disam_format)
-    local disam_str = disam_format:output(inlines, nil)
-    -- util.debug("update: " .. ir_.cite_item.id .. ": " .. disam_str)
-    ir_.disam_str = disam_str
-    if not engine.cite_irs_by_output[disam_str] then
-      engine.cite_irs_by_output[disam_str] = {}
-    end
-    engine.cite_irs_by_output[disam_str][ir_.ir_index] = ir_
-
+    -- DisamStringFormat doesn't render YearSuffix and this can be skipped.
+    -- local inlines = ir_:flatten(disam_format)
+    -- local disam_str = disam_format:output(inlines, nil)
+    -- util.debug(string.format('update %s: "%s" to "%s"', ir_.cite_item.id, ir_.disam_str, disam_str))
+    -- ir_.disam_str = disam_str
+    -- if not engine.cite_irs_by_output[disam_str] then
+    --   engine.cite_irs_by_output[disam_str] = {}
+    -- end
+    -- engine.cite_irs_by_output[disam_str][ir_.ir_index] = ir_
   end
 
   cite_ir.is_ambiguous = false
@@ -987,15 +1180,15 @@ local function find_first_names_ir(ir)
   end
 
   local first_rendering_ir = find_first(ir, function (ir_)
-    return (ir_._element == "text"
-      or ir_._element == "date"
-      or ir_._element == "number"
-      or ir_._element == "names"
-      or ir_._element == "label")
-      and ir_.group_var ~= "missing"
+    return (ir_._element_name == "text"
+      or ir_._element_name == "date"
+      or ir_._element_name == "number"
+      or ir_._element_name == "names"
+      or ir_._element_name == "label")
+      and ir_.group_var ~= GroupVar.Missing
   end)
   local first_names_ir
-  if first_rendering_ir and first_rendering_ir._element == "names" then
+  if first_rendering_ir and first_rendering_ir._element_name == "names" then
     first_names_ir = first_rendering_ir
   end
   if first_names_ir then
@@ -1058,12 +1251,12 @@ end
 -- See discretionary_AuthorOnly.txt
 function Citation:_apply_cite_author_only(ir)
   local author_ir = find_first(ir, function (ir_)
-    return (ir_._element == "text"
-      or ir_._element == "date"
-      or ir_._element == "number"
-      or ir_._element == "names"
-      or ir_._element == "label")
-      and ir_.group_var ~= "missing"
+    return (ir_._element_name == "text"
+      or ir_._element_name == "date"
+      or ir_._element_name == "number"
+      or ir_._element_name == "names"
+      or ir_._element_name == "label")
+      and ir_.group_var ~= GroupVar.Missing
   end)
 
   if author_ir then
@@ -1088,7 +1281,7 @@ function Citation:_apply_composite(ir, output_format, engine)
   -- local first_names_ir = find_first_names_ir(ir)
 
   local first_names_ir = find_first_names_ir(ir)
-  if first_names_ir then
+  if first_names_ir and engine.style.class ~= "note" then
     -- util.debug(first_names_ir)
     first_names_ir.collapse_suppressed = true
   end
@@ -1117,7 +1310,7 @@ function Citation:group_cites(irs)
     local first_names_ir = ir.first_names_ir
     if not first_names_ir then
       first_names_ir = find_first(ir, function (ir_)
-        return ir_._element == "names" and ir_.group_var ~= "missing"
+        return ir_._element_name == "names" and ir_.group_var ~= GroupVar.Missing
       end)
       if first_names_ir then
         local inlines = first_names_ir:flatten(disam_format)
@@ -1212,7 +1405,7 @@ function Citation:get_only_citation_number(ir)
   end
   local only_citation_number_ir
   for _, child in ipairs(ir.children) do
-    if child.group_var ~= "missing" then
+    if child.group_var ~= GroupVar.Missing then
       local citation_number_ir = self:get_only_citation_number(child)
       if citation_number_ir then
         if only_citation_number_ir then
@@ -1257,25 +1450,17 @@ function Citation:collapse_cites_by_year(irs)
         if i == #cite_group then
           cite_ir.own_delimiter = self.after_collapse_delimiter
         elseif i < #cite_group then
-          -- The delimiter depends on the citation > sort.
-          -- https://github.com/citation-style-language/test-suite/issues/39#issuecomment-687901688
-          if cite_ir.cite_item.locator then
-            -- Special hack for
-            cite_ir.own_delimiter = self.after_collapse_delimiter
-          elseif self.cite_grouping then
-            if self.sort then
-              cite_ir.own_delimiter = self.cite_group_delimiter
+          if self.style.class == "in-text" then
+            if cite_ir.cite_item.locator then
+              -- Special hack
+              cite_ir.own_delimiter = self.after_collapse_delimiter
             else
-              cite_ir.own_delimiter = self.layout.delimiter
+              cite_ir.own_delimiter = self.cite_group_delimiter or self.layout.delimiter
             end
           else
-            if self.sort then
-              cite_ir.own_delimiter = self.cite_group_delimiter
-            else
-              -- disambiguate_YearCollapseWithInstitution.txt
-              -- disambiguate_InitializeWithButNoDisambiguation.txt ?
-              cite_ir.own_delimiter = self.layout.delimiter
-            end
+            -- In note style, the layout delimiter is tried first before falling back to the default.
+            -- See <https://github.com/citation-style-language/test-suite/issues/56>
+            cite_ir.own_delimiter = self.layout.delimiter or self.cite_group_delimiter
           end
         end
       end
@@ -1289,7 +1474,7 @@ local function find_rendered_year_suffix(ir)
   end
   if ir.children then
     for _, child in ipairs(ir.children) do
-      if child.group_var ~= "missing" then
+      if child.group_var ~= GroupVar.Missing then
         local year_suffix = find_rendered_year_suffix(child)
         if year_suffix then
           return year_suffix
@@ -1357,7 +1542,7 @@ function Citation:suppress_ir_except_child(ir, target)
   ir.collapse_suppressed = true
   if ir.children then
     for _, child in ipairs(ir.children) do
-      if child.group_var ~= "missing" and not child.collapse_suppressed then
+      if child.group_var ~= GroupVar.Missing and not child.collapse_suppressed then
         if not self:suppress_ir_except_child(child, target) then
           ir.collapse_suppressed = false
         end
