@@ -1,21 +1,47 @@
 --
--- Copyright (c) 2021-2022 Zeping Lee
+-- Copyright (c) 2021-2024 Zeping Lee
 -- Released under the MIT license.
 -- Repository: https://github.com/zepinglee/citeproc-lua
 --
 
 local sort = {}
 
-local unicode = require("unicode")
+local uca_languages
+local uca_ducet
+local uca_collator
 
-local Element = require("citeproc-element").Element
-local Date = require("citeproc-node-date").Date
-local names = require("citeproc-node-names")
-local InlineElement = require("citeproc-output").InlineElement
-local util = require("citeproc-util")
+local element
+local output
+local util
+local node_date
+
+if kpse then
+  uca_languages = require("lua-uca-languages")
+  uca_ducet = require("lua-uca-ducet")
+  uca_collator = require("lua-uca-collator")
+  element = require("citeproc-element")
+  output = require("citeproc-output")
+  util = require("citeproc-util")
+  node_date = require("citeproc-node-date")
+else
+  uca_languages = require("citeproc.lua-uca.languages")
+  uca_ducet = require("citeproc.lua-uca.ducet")
+  uca_collator = require("citeproc.lua-uca.collator")
+  element = require("citeproc.element")
+  output = require("citeproc.output")
+  util = require("citeproc.util")
+  node_date = require("citeproc.node-date")
+end
+
+local Element = element.Element
+local Date = node_date.Date
+local InlineElement = output.InlineElement
 
 
 -- [Sorting](https://docs.citationstyles.org/en/stable/specification.html#sorting)
+---@class Sort: Element
+---@field children Key[]
+---@field sort_directions boolean[]
 local Sort = Element:derive("sort")
 
 function Sort:from_node(node)
@@ -46,13 +72,8 @@ function Sort:sort(items, state, context)
   if not Sort.collator then
     local lang = context.engine.lang
     local language = string.sub(lang, 1, 2)
-    -- It's 6 seconds slower to run the whole test-suite if these package
-    -- loading statements are put in the header.
-    local uca_ducet = require("lua-uca.lua-uca-ducet")
-    local uca_collator = require("lua-uca.lua-uca-collator")
     Sort.collator = uca_collator.new(uca_ducet)
     if language ~= "en" then
-      local uca_languages = require("lua-uca.lua-uca-languages")
       if uca_languages[language] then
         Sort.collator = uca_languages[language](Sort.collator)
       else
@@ -133,6 +154,14 @@ function Sort.compare_entry(key_map, sort_directions, item1, item2)
   end
 end
 
+
+---@class Key: Element
+---@field sort string?
+---@field variable string?
+---@field macro string?
+---@field names_min number?
+---@field names_use_first number?
+---@field names_use_last number?
 local Key = Element:derive("key")
 
 function Key:new()
